#include "lista.h"

void mostrarLista(GlobRef g)
{
	NodoRef p;
	if (NULL == g->ppio)
	{
		printf("\a\nNo hay nada que mostrar, lista vaca\n\n");
		return;
	}
	printf("\n+--------------+-----------------+-------------------+\n");
	printf(  "| Puntero      | Valor           | Siguiente         |\n");
	printf(  "+--------------+-----------------+-------------------+\n");

	for(p = g->ppio; p; p=p->sig)
		printf("| p = %8p | p->edad  = %4d | p->sig = %8p |\n",
				p, p->edad, p->sig);
	printf("+--------------+-----------------+-------------------+\n\n");
	return;
}

NodoRef prepararNodo()
{
	NodoRef nuevo = malloc(sizeof(Nodo));
	nuevo->edad = pedirInt(	"Deme una nmero:", -1000,1000);
	nuevo->sig = NULL;
	return nuevo;
}

void ponerPrincipio(GlobRef g)
{
	g->nodoPuesto = prepararNodo();
	if (NULL == g->nodoPuesto)
	{
		printf("\a\nIntento de aadir un nodo NULL\n\n");
		return;
	}
	if (NULL == g->ppio)
	{
		/* Lista vaca */
		g->ppio = g->fin = g->nodoPuesto;
		return;
	}
	/* Lista con uno o ms nodos */
	g->nodoPuesto->sig = g->ppio;
	g->ppio = g->nodoPuesto;
	return;
}

void ponerFinal(GlobRef g)
{
	g->nodoPuesto = prepararNodo();
	if (NULL == g->nodoPuesto)
	{
		printf("\a\nIntento de aadir un nodo NULL\n\n");
		return;
	}
	if (NULL == g->ppio)
	{
		g->ppio = g->nodoPuesto;
		g->fin  = g->nodoPuesto;
	}
	else
	{
		g->fin->sig = g->nodoPuesto;
		g->fin = g->nodoPuesto;
	}
	return;
}

void quitarPrincipio(GlobRef g)
{
	if (NULL == g->ppio)
	{
		printf("\a\nQuitar principio - lista vaca!\n\n");
		return;
	}
	if (g->ppio == g->fin)
	{
		g->nodoQuitado = g->ppio;
		g->ppio = g->fin = NULL;
		return;
	}
	/*
		Al menos hay dos elementos, ppio y fin
	 */
	g->nodoQuitado      = g->ppio;
	g->ppio             = g->ppio->sig;
	g->nodoQuitado->sig = NULL;
	return;
}

void quitarFinal(GlobRef g)
{
	NodoRef indice;
	if (NULL == g->ppio)
	{
		printf("\a\nQuitar final - lista vaca!\n\n");
		return;
	}
	if (g->ppio == g->fin)
	{
		g->nodoQuitado = g->ppio;
		g->ppio = g->fin = NULL;
		return;
	}
	/*
		Al menos hay dos elementos, ppio y fin.
		Hay que buscar el puntero del penltimo nodo.
	 */
	indice = g->ppio;
	while (indice->sig != g->fin)
		indice = indice->sig;
	g->nodoQuitado = g->fin;
	g->fin = indice;
	g->fin->sig = NULL;
	return;
}

void vaciarLista(GlobRef g)
{
	if (NULL == g->ppio)
	{
		printf("\a\nVaciar lista - lista vaca!\n\n");
		return;
	}
	while(NULL != g->ppio)
	{
		if (g->ppio == g->fin)
		{
			g->nodoQuitado = g->ppio;
			g->ppio = g->fin = NULL;
			return;
		}
		/*
				Al menos hay dos elementos, ppio y fin
		 */
		g->nodoQuitado      = g->ppio;
		g->ppio             = g->ppio->sig;
		g->nodoQuitado->sig = NULL;
	}
	return;
}
int menor_o_igual(NodoRef a, NodoRef b)
{
	return a->edad <= b->edad;
}
int mayor_o_igual(NodoRef a, NodoRef b)
{
	return a->edad >= b->edad;
}
void insertarOrdenado(GlobRef g)
{
	NodoRef nuevo = prepararNodo();
	NodoRef anter,sigui;
	if (NULL == g->ppio)
	{
		g->ppio = nuevo;
		g->fin = nuevo;
		return;
	}
	if (menor_o_igual(nuevo,g->ppio))
	{
		nuevo->sig = g->ppio;
		g->ppio    = nuevo;
		return;
	}
	if ( mayor_o_igual(nuevo,g->fin))
	{
		g->fin->sig = nuevo;
		g->fin = nuevo;
		return;
	}
	/* El nodo nuevo debe ocupar una posicin intermedia */
	anter  = g->ppio;
	sigui = anter->sig;
	while(!  (menor_o_igual(anter,nuevo) && menor_o_igual(nuevo,sigui)))
	{
		anter  = sigui;
		sigui = sigui->sig;
	}
	nuevo->sig = anter->sig;
	anter->sig = nuevo;
	return;
}

void borrarNodo(GlobRef g)
{
	int valor = pedirInt(	"Deme el valor que quiera borrar: ",
			-1000,
			1000);
	NodoRef indice    = NULL;
	NodoRef nodoHallado   = NULL;
	int presente = 0;
	for(indice = g->ppio; indice; indice = indice->sig)
		if (valor == indice->edad)
		{
			presente = 1;
			nodoHallado = indice;
			break;
		}
	if (presente)
	{
		if(nodoHallado == g->ppio)
		{
			quitarPrincipio(g);
			return;
		}
		if (nodoHallado == g->fin)
		{
			quitarFinal(g);
			return;
		}
		for(indice=g->ppio;indice;indice = indice->sig)
		{
			if (indice->sig == nodoHallado)
				break;
		}
		indice->sig = nodoHallado->sig;
		free(nodoHallado);
	}
	else
	{
		printf("\nEl nmero %d no consta en la lista.\n",valor);
	}
	return;
}
